/* 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Dinesh Nadarajah <n_dinesh@yahoo.com>
 *
 */

#include <config.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include <gtk/gtkintl.h>
//#include "gtkintl.h"
#include <gtk/gtkimcontextsimple.h>
#include <gtk/gtkimmodule.h>

GType type_tamilvp_tsc = 0;

static void tamilvp_tsc_class_init (GtkIMContextSimpleClass *class);
static void tamilvp_tsc_init (GtkIMContextSimple *im_context);

static void
tamilvp_tsc_register_type (GTypeModule *module)
{
  static const GTypeInfo object_info =
  {
    sizeof (GtkIMContextSimpleClass),
    (GBaseInitFunc) NULL,
    (GBaseFinalizeFunc) NULL,
    (GClassInitFunc) tamilvp_tsc_class_init,
    NULL,           /* class_finalize */
    NULL,           /* class_data */
    sizeof (GtkIMContextSimple),
    0,
    (GInstanceInitFunc) tamilvp_tsc_init,
  };

  type_tamilvp_tsc = 
    g_type_module_register_type (module,
				 GTK_TYPE_IM_CONTEXT_SIMPLE,
				 "GtkIMContextTamilVPUni",
				 &object_info, 0);
}

#define SYL1(a,b,c,d,e,f)  a+128, b, c, d, e, f
#define SYL2(a,b,c,d,e,f)  a+128, b+128, c, d, e, f
#define SYL3(a,b,c,d,e,f)  a+128, b+128, c+128, d, e, f
#define SYL4(a,b,c,d,e,f)  a+128, b+128, c+128, d+128, e, f


/* The sequences here match the sequences used in the emacs quail
 * mode cryllic-translit; they allow entering all characters
 * in iso-8859-5
 */
static guint16 tamil_compose_seqs[] = {

  SYL1(GDK_0,	0,		0,		0,		0,		0x30),
  SYL1(GDK_1,	0,		0,		0,		0,		0x31),
  SYL1(GDK_2,	0,		0,		0,		0,		0x32),
  SYL1(GDK_3,	0,		0,		0,		0,		0x33),
  SYL1(GDK_4,	0,		0,		0,		0,		0x34),
  SYL1(GDK_5,	0,		0,		0,		0,		0x35),
  SYL1(GDK_6,	0,		0,		0,		0,		0x36),
  SYL1(GDK_7,	0,		0,		0,		0,		0x37),
  SYL1(GDK_8,	0,		0,		0,		0,		0x38),
  SYL1(GDK_9,	0,		0,		0,		0,		0x39),

  SYL1(GDK_A,	0,		0,		0,		0,		0xab),
  SYL2(GDK_A,	GDK_A,	0,		0,		0,		0xac),
  SYL2(GDK_A,	GDK_I,	0,		0,		0,		0xb3),
  SYL2(GDK_A,	GDK_U,	0,		0,		0,		0xb6),

  SYL1(GDK_B,	0,		0,		0,		0,		0x00),

  SYL1(GDK_C,	0,		0,		0,		0,		0xbb),
  SYL2(GDK_C,	GDK_q,	0,		0,		0,		0xef),
  SYL2(GDK_C,	GDK_u,	0,		0,		0,		0x9a),
  SYL3(GDK_C,	GDK_u,	GDK_u,	0,		0,		0x9c),

  SYL1(GDK_D,	0,		0,		0,		0,		0x00),

  SYL1(GDK_E,	0,		0,		0,		0,		0xb1),
  SYL2(GDK_E,	GDK_E,	0,		0,		0,		0xb2),
  SYL1(GDK_F,	0,		0,		0,		0,		0x00),
  SYL1(GDK_G,	0,		0,		0,		0,		0x00),
  SYL1(GDK_H,	0,		0,		0,		0,		0x00),
  SYL1(GDK_I,	0,		0,		0,		0,		0xfe),
  SYL2(GDK_I,	GDK_I,	0,		0,		0,		0xae),
  SYL1(GDK_J,	0,		0,		0,		0,		0x00),
  SYL1(GDK_K,	0,		0,		0,		0,		0x00),

  SYL1(GDK_L,	0,		0,		0,		0,		0xc7),
  SYL2(GDK_L,	GDK_q,	0,		0,		0,		0xfb),
  SYL2(GDK_L,	GDK_u,	0,		0,		0,		0xd9),
  SYL3(GDK_L,	GDK_u,	GDK_u,	0,		0,		0xe9),

  SYL1(GDK_M,	0,		0,		0,		0,		0x00),

  SYL1(GDK_N,	0,		0,		0,		0,		0xbd),
  SYL2(GDK_N,	GDK_q,	0,		0,		0,		0xf1),
  SYL2(GDK_N,	GDK_u,	0,		0,		0,		0xcf),
  SYL3(GDK_N,	GDK_u,	GDK_u,	0,		0,		0xdf),

  SYL1(GDK_O,	0,		0,		0,		0,		0xb4),
  SYL2(GDK_O,	GDK_O,	0,		0,		0,		0xb5),
  SYL1(GDK_P,	0,		0,		0,		0,		0x00),
  SYL1(GDK_Q,	0,		0,		0,		0,		0xb7),

  SYL1(GDK_R,	0,		0,		0,		0,		0xc8),
  SYL2(GDK_R,	GDK_q,	0,		0,		0,		0xfc),
  SYL2(GDK_R,	GDK_u,	0,		0,		0,		0xda),
  SYL3(GDK_R,	GDK_u,	GDK_u,	0,		0,		0xea),

  SYL1(GDK_S,	0,		0,		0,		0,		0x84),
  SYL2(GDK_S,	GDK_q,	0,		0,		0,		0x89),
  SYL1(GDK_T,	0,		0,		0,		0,		0x00),
  SYL1(GDK_U,	0,		0,		0,		0,		0xaf),
  SYL2(GDK_U,	GDK_U,	0,		0,		0,		0xb0),
  SYL1(GDK_V,	0,		0,		0,		0,		0x00),
  SYL1(GDK_W,	0,		0,		0,		0,		0xb3),
  SYL1(GDK_X,	0,		0,		0,		0,		0x00),
  SYL1(GDK_Y,	0,		0,		0,		0,		0x00),
  SYL1(GDK_Z,	0,		0,		0,		0,		0x00),

  SYL1(GDK_a,	0,		0,		0,		0,		0xa1),
  SYL2(GDK_a,	GDK_i,	0,		0,		0,		0xb3),
  SYL2(GDK_a,	GDK_u,	0,		0,		0,		0xaa),

  SYL1(GDK_b,	0,		0,		0,		0,		0xbf),
  SYL2(GDK_b,	GDK_q,	0,		0,		0,		0xf3),
  SYL2(GDK_b,	GDK_u,	0,		0,		0,		0xd1),
  SYL3(GDK_b,	GDK_u,	GDK_u,	0,		0,		0xe1),

  SYL1(GDK_c,	0,		0,		0,		0,		0xba),
  SYL2(GDK_c,	GDK_q,	0,		0,		0,		0xee),
  SYL2(GDK_c,	GDK_u,	0,		0,		0,		0xcd),
  SYL3(GDK_c,	GDK_u,	GDK_u,	0,		0,		0xdd),

  SYL1(GDK_d,	0,		0,		0,		0,		0xbe),
  SYL2(GDK_d,	GDK_q,	0,		0,		0,		0xf2),
  SYL2(GDK_d,	GDK_u,	0,		0,		0,		0xd0),
  SYL3(GDK_d,	GDK_u,	GDK_u,	0,		0,		0xe0),

  SYL1(GDK_e,	0,		0,		0,		0,		0xa6),
  SYL2(GDK_e,	GDK_e,	0,		0,		0,		0xa7),
  SYL3(GDK_e,	GDK_e,	GDK_e,	0,		0,		0xb1),
  SYL4(GDK_e,	GDK_e,	GDK_e,	GDK_e,	0,		0xb2),

  SYL1(GDK_f,	0,		0,		0,		0,		0xaa),

  SYL1(GDK_g,	0,		0,		0,		0,		0xb9),
  SYL2(GDK_g,	GDK_q,	0,		0,		0,		0xed),
  SYL2(GDK_g,	GDK_u,	0,		0,		0,		0x99),
  SYL3(GDK_g,	GDK_u,	GDK_u,	0,		0,		0x9b),

  SYL1(GDK_h,	0,		0,		0,		0,		0x86),
  SYL2(GDK_h,	GDK_q,	0,		0,		0,		0x8b),

  SYL1(GDK_i,	0,		0,		0,		0,		0xa2),
  SYL2(GDK_i,	GDK_i,	0,		0,		0,		0xa3),

  SYL1(GDK_j,	0,		0,		0,		0,		0x83),
  SYL2(GDK_j,	GDK_q,	0,		0,		0,		0x88),

  SYL1(GDK_k,	0,		0,		0,		0,		0xb8),
  SYL2(GDK_k,	GDK_q,	0,		0,		0,		0xec),
  SYL2(GDK_k,	GDK_u,	0,		0,		0,		0xcc),
  SYL3(GDK_k,	GDK_u,	GDK_u,	0,		0,		0xdc),

  SYL1(GDK_l,	0,		0,		0,		0,		0xc4),
  SYL2(GDK_l,	GDK_q,	0,		0,		0,		0xf8),
  SYL2(GDK_l,	GDK_u,	0,		0,		0,		0xd6),
  SYL3(GDK_l,	GDK_u,	GDK_u,	0,		0,		0xe6),

  SYL1(GDK_m,	0,		0,		0,		0,		0xc1),
  SYL2(GDK_m,	GDK_q,	0,		0,		0,		0xf5),
  SYL2(GDK_m,	GDK_u,	0,		0,		0,		0xd3),
  SYL3(GDK_m,	GDK_u,	GDK_u,	0,		0,		0xe3),

  SYL1(GDK_n,	0,		0,		0,		0,		0xc9),
  SYL2(GDK_n,	GDK_g,	0,		0,		0,		0xb9),
  SYL3(GDK_n,	GDK_g,	GDK_q,	0,		0,		0xed),
  SYL3(GDK_n,	GDK_g,	GDK_u,	0,		0,		0x99),
  SYL4(GDK_n,	GDK_g,	GDK_u,	GDK_u,	0,		0x9b),
  SYL2(GDK_n,	GDK_j,	0,		0,		0,		0xbb),
  SYL3(GDK_n,	GDK_j,	GDK_q,	0,		0,		0xef),
  SYL3(GDK_n,	GDK_j,	GDK_u,	0,		0,		0x9a),
  SYL4(GDK_n,	GDK_j,	GDK_u,	GDK_u,	0,		0x9c),
  SYL2(GDK_n,	GDK_q,	0,		0,		0,		0xfd),
  SYL2(GDK_n,	GDK_u,	0,		0,		0,		0xdb),
  SYL3(GDK_n,	GDK_u,	GDK_u,	0,		0,		0xeb),

  SYL1(GDK_o,	0,		0,		0,		0,		0xa6),
  SYL2(GDK_o,	GDK_o,	0,		0,		0,		0xa7),

  SYL1(GDK_p,	0,		0,		0,		0,		0xc0),
  SYL2(GDK_p,	GDK_q,	0,		0,		0,		0xf4),
  SYL2(GDK_p,	GDK_u,	0,		0,		0,		0xd2),
  SYL3(GDK_p,	GDK_u,	GDK_u,	0,		0,		0xe2),

  SYL1(GDK_q,	0,		0,		0,		0,		0),

  SYL1(GDK_r,	0,		0,		0,		0,		0xc3),
  SYL2(GDK_r,	GDK_q,	0,		0,		0,		0xf7),
  SYL2(GDK_r,	GDK_u,	0,		0,		0,		0xd5),
  SYL3(GDK_r,	GDK_u,	GDK_u,	0,		0,		0xe5),

  SYL1(GDK_s,	0,		0,		0,		0,		0x85),
  SYL2(GDK_s,	GDK_h,	0,		0,		0,		0x84), 
  SYL3(GDK_s,	GDK_h,	GDK_q,	0,		0,		0x89), 
  SYL3(GDK_s,	GDK_h,	GDK_r,	0,		0,		0x82), 
  SYL4(GDK_s,	GDK_h,	GDK_r,	GDK_i,	0,		0x82),
  SYL2(GDK_s,	GDK_q,	0,		0,		0,		0x8a),

  SYL1(GDK_t,	0,		0,		0,		0,		0xbc),
  SYL2(GDK_t,	GDK_h,	0,		0,		0,		0xbe),
  SYL3(GDK_t,	GDK_h,	GDK_q,	0,		0,		0xf2),
  SYL3(GDK_t,	GDK_h,	GDK_u,	0,		0,		0xd0),
  SYL4(GDK_t,	GDK_h,	GDK_u,	GDK_u,	0,		0xe0),
  SYL2(GDK_t,	GDK_i,	0,		0,		0,		0xca),
  SYL3(GDK_t,	GDK_i,	GDK_i,	0,		0,		0xcb),
  SYL2(GDK_t,	GDK_q,	0,		0,		0,		0xf0),
  SYL2(GDK_t,	GDK_u,	0,		0,		0,		0xce),
  SYL3(GDK_t,	GDK_u,	GDK_u,	0,		0,		0xde),

  SYL1(GDK_u,	0,		0,		0,		0,		0xa4),
  SYL2(GDK_u,	GDK_u,	0,		0,		0,		0xa5),

  SYL1(GDK_v,	0,		0,		0,		0,		0xc5),
  SYL2(GDK_v,	GDK_q,	0,		0,		0,		0xf9),
  SYL2(GDK_v,	GDK_u,	0,		0,		0,		0xd7),
  SYL3(GDK_v,	GDK_u,	GDK_u,	0,		0,		0xe7),

  SYL1(GDK_w,	0,		0,		0,		0,		0xa8),

  SYL1(GDK_x,	0,		0,		0,		0,		0x87),
  SYL2(GDK_x,	GDK_q,	0,		0,		0,		0x8c),

  SYL1(GDK_y,	0,		0,		0,		0,		0xc2),
  SYL2(GDK_y,	GDK_q,	0,		0,		0,		0xf6),
  SYL2(GDK_y,	GDK_u,	0,		0,		0,		0xd4),
  SYL3(GDK_y,	GDK_u,	GDK_u,	0,		0,		0xe4),

  SYL1(GDK_z,	0,		0,		0,		0,		0xc6),
  SYL2(GDK_z,	GDK_q,	0,		0,		0,		0xfa),
  SYL2(GDK_z,	GDK_u,	0,		0,		0,		0xd8),
  SYL3(GDK_z,	GDK_u,	GDK_u,	0,		0,		0xe8)
};

static void
tamilvp_tsc_class_init (GtkIMContextSimpleClass *class)
{
}

static void
tamilvp_tsc_init (GtkIMContextSimple *im_context)
{
  gtk_im_context_simple_add_table (im_context,
				   tamil_compose_seqs,
				   4,
				   G_N_ELEMENTS (tamil_compose_seqs) / (4 + 2));
}

static const GtkIMContextInfo tamilvp_tsc_info = { 
  "tamilvp_tsc",		   /* ID */
  N_("TamilVP (TSCII)"), /* Human readable name */
  "gtk+",			   /* Translation domain */
   "",		   /* Dir for bindtextdomain (not strictly needed for "gtk+") */
  "ta_IN"			           /* Languages for which this module is the default */
};

static const GtkIMContextInfo *info_list[] = {
  &tamilvp_tsc_info
};

void
im_module_init (GTypeModule *module)
{
  tamilvp_tsc_register_type (module);
}

void 
im_module_exit (void)
{
}

void 
im_module_list (const GtkIMContextInfo ***contexts,
		int                      *n_contexts)
{
  *contexts = info_list;
  *n_contexts = G_N_ELEMENTS (info_list);
}

GtkIMContext *im_module_create (const gchar *context_id)
{
  if (strcmp (context_id, "tamilvp_tsc") == 0)
    return GTK_IM_CONTEXT (g_object_new (type_tamilvp_tsc, NULL));
  else
    return NULL;
}
